#!/usr/bin/env python
# filename: mangodb.py


#
# Copyright (c) 2015 Bryan Briney
# License: The MIT license (http://opensource.org/licenses/MIT)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software
# and associated documentation files (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge, publish, distribute,
# sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or
# substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
# BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
# DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#


from __future__ import absolute_import, division, print_function, unicode_literals

import logging
import platform
import os
import subprocess as sp
import sys

from pymongo import MongoClient

from . import log

if sys.version_info[0] > 2:
    STR_TYPES = [str, ]
else:
    STR_TYPES = [str, unicode]










def list_dbs(ip='localhost', port=27017, user=None, connect=True, hint=None):
    '''
    Returns a list of Databases.
    .. note:
        Both ``user`` and ``password`` are required when connecting to a MongoDB
        database that has authentication enabled.
    Arguments:
        db (str): Name of the MongoDB database. Required.
        ip (str): IP address of the MongoDB server. Default is ``localhost``.
        port (int): Port of the MongoDB server. Default is ``27017``.
        user (str): Username, if authentication is enabled on the MongoDB database.
            Default is ``None``, which results in requesting the connection
            without authentication.
        password (str): Password, if authentication is enabled on the MongoDB database.
            Default is ``None``, which results in requesting the connection
            without authentication.
            
        hint (str): substring found in database name, if used only list of 
            databases that contain the substring will returned
    '''
    
    #Darwin says "you shall not connect!"
    if platform.system().lower() == 'darwin':
        connect = False
    else:
        connect = True
    
    # check for user and pass, if so: make uri, import urllib, return dbs
    if user and password:
        import urllib
        pwd = urllib.quote_plus(password)
        uri = 'mongodb://{}:{}@{}:{}'.format(user, pwd, ip, port)
        #check to see if hint was passed in and return list of dbs with hint
        if hint and type(hint) == str:
            ls = MongoClient(uri, connect=connect).list_database_names()
            return [d for d in ls if hint.upper() in d.upper()]
        #otherwise return a list of all dbs
        else:
            return MongoClient(uri, connect=connect).list_database_names()
        
    #otherwise no user or pass required    
    else:
        #check to see if hint was passed in and return list of dbs with hint
        if hint and type(hint) == str:
            ls = MongoClient(ip, port, connect=connect).list_database_names()
            return [d for d in ls if hint.upper() in d.upper()]
        #otherwise return a list of all dbs
        else:
            return MongoClient(ip, port, connect=connect).list_database_names()


#!/usr/bin/env python
# filename: df.py

import itertools
import os
import sys

import numpy as np
import pandas as pd

import matplotlib.pyplot as plt
import seaborn as sns

from abutils.utils import mongodb
from abutils.utils import color

import pymongo

from natsort import natsorted


def get_freq_df(db, collections, value, chain='heavy', match=None, normalize = False):

    # database
    if type(db) == pymongo.database.Database:
        DB = db
    elif type(db) == str:
        DB = mongodb.get_db(db)
    else:
        print ('Database not correct')
        return
    
    if type(collections) == list:
        collections = collections
    else:
        collections = [collections, ]
    
    if not match:
        match = {'chain': chain, 'prod': 'yes'}
    group = {'_id': '${}'.format(value), 'count': {'$sum': 1}}

    # initialize a dictionary that will hold all of the DataFrames we're making (one per subject)
    data = {}

        
    # iterate through each of the collections in the subject group
    for collection in collections:
        #print(collection)
        data[collection] = {}

        # get the aggregation data from MongoDB
        res = DB[collection].aggregate([{'$match': match}, {'$group': group}])

        # convert the MongoDB aggregation result into a dictionary iof V-gene counts
        for r in res:
            data[collection][r['_id']] = r['count']

    #print('')

    # construct a DataFrame from the dictionary of V-gene counts
    df = pd.DataFrame(data)
    
    if normalize:
        df = df / df.sum()
        df = df.dropna(0)

    return df


#!/usr/bin/env python
# filename: plot.py

import itertools
import os
import sys

import numpy as np
import pandas as pd

import matplotlib.pyplot as plt
import seaborn as sns

from abutils.utils import mongodb
from abutils.utils import color

import pymongo
from tqdm import tqdm

from natsort import natsorted

from .df import get_freq_df

def df_freq_barplot(df, cols=None):

    #get the columns
    if not cols:
        cols = list(df.columns)

    #melt the df
    df = df.melt(id_vars='index', value_vars=cols)

    #plot
    sns.barplot(data = df, x='index', y='value', hue='variable')
    
    
def QuickDataCheck(db, collections=None, index=False, values=None, match=None):
    # This function will quickly allow you to check the sequencing data of a database
    # database
    if type(db) == pymongo.database.Database:
        DB = db
    elif type(db) == str:
        DB = mongodb.get_db(db)
    else:
        print ('Database not correct')
        return
    
    if collections is None:
        colls = mongodb.get_collections(DB)
    else:
        colls = collections
    
    #index the collections if applicable 
    if index:
        print('Indexing Collections...')
        for collection in tqdm(colls):
            DB[collection].create_index([('chain', 1),
                                         ('prod', 1), 
                                         ('v_gene.gene', 1), 
                                         ('cdr3_len', 1)], 
                                        name='productive heavychain cdr3_len', 
                                        default_language='english')
    #if there is a set values, then use those
    if values:
        print('Getting data...')
        dfs = [get_freq_df(DB, colls, value, normalize=True, match=match) for value in values]

    else:
        print('Getting data...')
        values = ['v_gene.gene', 'cdr3_len']
        dfs = [get_freq_df(DB, colls, value, normalize=True, match=match) for value in values]
    
    #now plot the figures for each value
    for df, value in zip(dfs, values):
        print('-----------')
        print(value)
        print('-----------')
        for collection in df.columns:
            print(collection)
            #Try to plot the value unless the requested value is invalid
            try:
                df2 = pd.DataFrame(df[collection]).reset_index().melt(id_vars='index', value_vars=df.columns)
                try:
                    fam = [d.split('-')[0] for d in df2['index']]
                    df2['fam'] = fam
                except AttributeError:
                    None

                plt.figure(figsize=[12,4])
                try:
                    g = sns.barplot(data = df2, x='index', y='value', hue='fam', dodge=False) 
                except ValueError:
                    g = sns.barplot(data = df2, x='index', y='value', dodge=False)
                try:
                    g.get_legend().remove()
                except AttributeError:
                    None
                plt.xticks(rotation=90)
                plt.tight_layout()
                plt.show()
                print(' ')
            except ValueError:
                print('The value you requested is in valid')




import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name='antibodies',
    version='1',
    #packages=['antibodies',],
    description='Helpful Scripts for Antibody NGS Data Processing',
    author="CollinJ0",
    url= 'https://www.github.com/CollinJ0/antibodies',
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    scripts=['bin/antibodies'],
    install_requires=[           
            'numpy',
            'pandas',
            'matplotlib',
            'seaborn',
            'pymongo',
            'tqdm',
            'abutils',
        ],
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
)